﻿//------------------------------------------------------------------------
// ● 使用ライブラリの宣言
//------------------------------------------------------------------------
using UnityEngine;
using System.Collections;
//========================================================================
// ■ BaseCharacter
//------------------------------------------------------------------------
//	キャラクターの基盤クラス。
//========================================================================

public class BaseCharacter : MonoBehaviour {
	//--------------------------------------------------------------------
	// ● メンバ変数
	//--------------------------------------------------------------------
	protected StateBase state;
	[HideInInspector] public CharacterController controller;
	[HideInInspector] public Animator animator;
	public float defaultMoveSpeed = 4;
	public float defaultRotateSpeed = 180;
	public float waitSecond = 5;
	[HideInInspector] public Vector3 moveVector;
	//--------------------------------------------------------------------
	// ● 状態を変更
	//--------------------------------------------------------------------
	public void ChangeState(StateBase newState) {
		// 前回状態を終了
		if (state != null)	state.MyFinalize();
		state = newState;		// 現在状態に適用
		state.ai = this;		// 自身を登録
		state.Initialize();		// 初期化
	}
	//--------------------------------------------------------------------
	// ● 初期化
	//--------------------------------------------------------------------
	protected virtual void Start() {
		transform.parent = null;

		controller = GetComponent<CharacterController>();
		animator = GetComponent<Animator>();
		ChangeState( new StateEternalWait() );
	}
	//--------------------------------------------------------------------
	// ● 更新
	//--------------------------------------------------------------------
	protected virtual void Update() {
		state.Update();

		controller.SimpleMove(moveVector);
		animator.SetFloat("MoveDistance", moveVector.magnitude);
	}
	//--------------------------------------------------------------------
	// ● 更新（テスト）
	//--------------------------------------------------------------------
	protected virtual void UpdateTest() {
		if ( Input.GetKeyDown(KeyCode.Return) ) {
			if ( state.GetType() == typeof(StateEternalWait) )
				ChangeState( new StateWait() );

			if ( state.GetType() == typeof(StateWait) )
				ChangeState( new StateMove() );

			else if ( state.GetType() == typeof(StateMove) )
				ChangeState( new StateFirstMove() );
			
			else if ( state.GetType() == typeof(StateFirstMove) )
				ChangeState( new StateRotateRight90() );
			
			else if ( state.GetType() == typeof(StateRotateRight90) )
				ChangeState( new StateRotateLeft90() );

			else if ( state.GetType() == typeof(StateRotateLeft90) )
				ChangeState( new StateRotate180() );

			else if ( state.GetType() == typeof(StateRotate180) )
				ChangeState( new StateWait() );
		}

		
//		Debug.Log(state);
	}
	//--------------------------------------------------------------------
	// ● 新たに物理衝突したコールバック関数
	//--------------------------------------------------------------------
//	virtual protected void OnCollisionEnter(Collision collision) {
//		Debug.Log("moto");
//	}
}