﻿using UnityEngine;
using System.Collections;

[RequireComponent(typeof(FieldSignCounter))]

public class Player : MonoBehaviour
{
    /*操作説明
	アローキーで移動
	X(将来はSPACE)で標識設置、標識回収??
	Q,Eで装備標識選択
	*/

    public float movedistance = 2;      //移動距離
	public float moveintervalframe = 30;//移動時間
	public GameObject[] hyousiki_rist;  //生成用
	public int hyousiki_number = 0;     //標識切り替え用
    CharacterController controller;
    Transform camera_;
    private bool isOverlaping;  //標識を重ねて設置する恐れがあるか
    private bool ismove = false;

    GameObject _overlapingObject;   //自分に重なって配置されてる標識

    StageGenerator _stageGenerator;
    FieldSignCounter _signCounter;

    Vector3 velocity;
    Vector3 startmoveposition;

	Animator animator;	// アニメータークラス

    // Use this for initialization
    void Start () {
		transform.parent = null;


		//CONTROLLER = GetComponents (CharacterController);

        controller = GetComponent<CharacterController>();
        camera_ = Camera.main.transform;

        _stageGenerator = camera_.GetComponent<StageGenerator>();
        _signCounter = GetComponent<FieldSignCounter>();

		// アニメーターを取得
		animator = GetComponent<Animator>();
    }
	
	// Update is called once per frame
	void FixedUpdate ()
    {
		//CONTROLLER.SimpleMove (Horizontal);

		// 回転処理
		Vector3 eulerAngles = transform.eulerAngles;
		eulerAngles.y = camera_.eulerAngles.y;
		transform.rotation = Quaternion.Euler(eulerAngles);

		// 移動処理
		//キー操作
		if (ismove == false) //移動中の操作制限
		{
			if (Input.GetKey (KeyCode.UpArrow))
            {
                var forward = new Vector3(0.0f, 0.0f, movedistance);

                if (!WallIsOnForward(transform.position, forward))
                {
                    velocity = new Vector3(0.0f, 0.0f, movedistance / moveintervalframe);
                    startmoveposition = transform.position;
                    ismove = true;
                }
			}
			else if (Input.GetKey (KeyCode.DownArrow))
            {
                var forward = new Vector3(0.0f, 0.0f, -movedistance);

                if (!WallIsOnForward(transform.position, forward))
                {
                    velocity = new Vector3(0.0f, 0.0f, -movedistance / moveintervalframe);
                    ismove = true;
                    startmoveposition = transform.position;
                }

            }
			if (Input.GetKey (KeyCode.LeftArrow))
            {
                var forward = new Vector3(-movedistance, 0.0f, 0.0f);

                if (!WallIsOnForward(transform.position, forward))
                {
                    velocity = new Vector3(-movedistance / moveintervalframe, 0.0f, 0f);
                    ismove = true;
                    startmoveposition = transform.position;
                }
			}
			else if (Input.GetKey (KeyCode.RightArrow))
            {
                var forward = new Vector3(movedistance, 0.0f, 0.0f);

                if (!WallIsOnForward(transform.position, forward))
                {
                    velocity = new Vector3(movedistance / moveintervalframe, 0.0f, 0f);
                    ismove = true;
                    startmoveposition = transform.position;
                }
			}

            //hyousikijenelate ();

            //オブジェクト配置処理
            if (Input.GetKeyDown(KeyCode.Space) || Input.GetKeyDown(KeyCode.Return) || Input.GetKeyDown(KeyCode.LeftShift))
            { // Xキー

                //オブジェクトに重なっているなら取り除く
                if (isOverlaping)
                    RemoveSign();
                //それ以外は生成
                else
                    CreateSign();
			}
		}
			
        //選択標識切り替え
		hyousikijenelate ();

		if (ismove) {			
			transform.position += velocity;
		}

		float currentDistance = (transform.position - startmoveposition).magnitude;
		if (movedistance <= currentDistance) {
			ismove = false;
        }
		
		// 移動アニメーションを設定
		animator.SetFloat("MoveDistance", ismove ? 1 : 0);
	}

	void hyousikijenelate()
    {
		if (Input.GetKeyDown (KeyCode.Q)) { // Qキー
            if (hyousiki_number > 0)
            {
                hyousiki_number--;
            }
		}
		if (Input.GetKeyDown (KeyCode.E)) { // Eキー
            if (hyousiki_number < hyousiki_rist.Length - 1)
            {
                hyousiki_number++;
            }
		}
	}

    //進もうとしている方向に壁があるか
    bool WallIsOnForward(Vector3 position, Vector3 forward)
    {
        RaycastHit[] hits = Physics.RaycastAll( new Ray(position, forward.normalized), forward.magnitude);

        foreach (var hit in hits)
        {
            if (hit.collider.name.Contains("Wall"))
                return true;
        }

        return false;
    }

    void CreateSign()
    {
        if (_signCounter._maxSignNumber <= _signCounter._currentSignNumber)
            return;

        var position = transform.position;

        //選択された標識を形成 Quaternion.identity
        var sign = Instantiate(hyousiki_rist[hyousiki_number], position, Quaternion.Euler(0,180,0)) as GameObject;
    }

    void RemoveSign()
    {
        if (_overlapingObject != null)
            Destroy(_overlapingObject);

        _overlapingObject = null;
        isOverlaping = false;
    }

    void OnTriggerStay(Collider coll)
    {
        if (coll.gameObject.tag != "TrafficSign")
            return;

        isOverlaping = true;
        _overlapingObject = coll.gameObject;
    }

    void OnTriggerExit(Collider coll)
    {
        isOverlaping = false;
        _overlapingObject = null;
    }
}